/*
  +-------------------------------------+
  |                                     |
  |   Code to handle Windows sockets.   |
  |                                     |
  +-------------------------------------+
*/
#ifndef SOCKS_DEF
#define SOCKS_DEF

//
// Socket exceptions:
//
const int WINSOCK_FAILURE = 1;
const int BAD_SOCKET = 2;
const int BIND_FAILURE = 3;
const int LISTEN_FAILURE = 4;
const int HOSTNAME_LOOKUP_FAILURE = 5;
const int IPADDRESS_LOOKUP_FAILURE = 6;
const int CONNECT_FAILURE = 7;
const int ACCEPT_FAILURE = 8;
const int SEND_FAILURE = 9;
const int RECV_FAILURE = 10;
const int GETPEERNAME_FAILURE = 11;

/*
  +------------------------------------------+
  |                                          |
  |   This class handles initialization of   |
  |   Windows sockets and guarantees that    |
  |   the WSACleanup function gets called    |
  |   when we exit.                          |
  |                                          |
  +------------------------------------------+
*/

class wsockets_t {
    int ok;
public:
    wsockets_t();
    ~wsockets_t();
};

/*
  +-----------------------------------------------------------+
  |                                                           |
  |   Socket class.  Automagically closes socket              |
  |   when it goes out of scope.  Hides some ugly details.    |
  |   I've simplified this somewhat in that I've combined     |
  |   bind() and listen() since I always want it to do a      |
  |   "bind" followed by a "listen" on the local host, etc.   |
  |   I've also removed parameters that I'm always going to   |
  |   set the same way--maybe later I will have to            |
  |   extend...                                               |
  |                                                           |
  +-----------------------------------------------------------+
*/
class socket_t {
  private:
    SOCKET sd;
  public:
    socket_t();
    socket_t( socket_t &socket)
    {
        sd = socket.sd;
        socket.sd = INVALID_SOCKET;
    }
    socket_t( SOCKET as) : sd( as) {}
    ~socket_t();
    void create();
    socket_t &operator=( socket_t &s);
    socket_t &operator=( SOCKET as);
    //int ok() { return sd != INVALID_SOCKET; }
    void listen( unsigned short port, int maxqlen);
    void connect( char *host, unsigned short port);
    SOCKET accept();
    void close();
    int send( const char *buf, int len)
    {
        int nwritten = ::send( sd, buf, len, 0);
        if( nwritten == SOCKET_ERROR ) throw SEND_FAILURE;
        return nwritten;
    }
    int recv( char *buf, int len)
    {
        int nread = ::recv( sd, buf, len, 0);
        if( nread == SOCKET_ERROR ) throw RECV_FAILURE;
        if( nread == 0 ) throw RECV_FAILURE;  // Is this really correct?
        return nread;
    }
    void getpeername( char *name, int nmsz);
};

/*
  +-----------------------------+
  |                             |
  |   Miscelaneous functions.   |
  |                             |
  +-----------------------------+
*/

//
// Return the IP address of the host given by hostname.
//  (Should I worry about the newer INET6 addresses???)
//
int get_ip_address( char *hostname, in_addr *inaddr);

//
// Print out local host name and address.
//
void print_localhost_info();

#endif
